defmodule Chronoscope.NTS.KeyEstablishmentTest do
  use ExUnit.Case

  alias Chronoscope.NTS.KeyEstablishment

  test "builds request" do
    assert KeyEstablishment.request() ==
             <<0x80, 0x01, 0x00, 0x02, 0x00, 0x00, 0x80, 0x04, 0x00, 0x04, 0x00, 0x1E, 0x00, 0x0F, 0x80, 0x00, 0x00,
               0x00>>
  end

  test "handles empty response" do
    assert KeyEstablishment.parse_response([]) == {:ok, %{cookies: 0}}
  end

  test "handles end of message record" do
    assert KeyEstablishment.parse_response([0x80, 0x00, 0x00, 0x00]) == {:ok, %{cookies: 0}}
  end

  test "handles next protocol negotiation response" do
    assert KeyEstablishment.parse_response([0x80, 0x01, 0x00, 0x02, 0x00, 0x00]) ==
             {:ok, %{cookies: 0, next_protocols: ["NTPv4"]}}
  end

  test "does not handle next protocol negotiation response without critical bit" do
    assert KeyEstablishment.parse_response([0x00, 0x01, 0x00, 0x02, 0x00, 0x00]) == {:ok, %{cookies: 0}}
  end

  test "handles empty next protocols" do
    assert KeyEstablishment.parse_response([0x80, 0x01, 0x00, 0x00]) == {:ok, %{cookies: 0, next_protocols: []}}
  end

  test "handles multiple next protocols" do
    assert KeyEstablishment.parse_response([0x80, 0x01, 0x00, 0x06, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00]) ==
             {:ok, %{cookies: 0, next_protocols: ["NTPv4", "UNASSIGNED", "NTPv4"]}}
  end

  test "handles aead algorithm negotiation response" do
    assert KeyEstablishment.parse_response([0x80, 0x04, 0x00, 0x02, 0x00, 0x0F]) ==
             {:ok, %{cookies: 0, aead_algorithms: ["AEAD_AES_SIV_CMAC_256"]}}
  end

  test "handles aead algorithm negotiation response without critical bit" do
    assert KeyEstablishment.parse_response([0x00, 0x04, 0x00, 0x02, 0x00, 0x1E]) ==
             {:ok, %{cookies: 0, aead_algorithms: ["AEAD_AES_128_GCM_SIV"]}}
  end

  test "handles empty aead algorithms" do
    assert KeyEstablishment.parse_response([0x80, 0x04, 0x00, 0x00]) == {:ok, %{cookies: 0, aead_algorithms: []}}
  end

  test "handles multiple aead algorithms" do
    assert KeyEstablishment.parse_response([0x80, 0x04, 0x00, 0x06, 0x00, 0x1E, 0x00, 0x01, 0x00, 0x0F]) ==
             {:ok, %{cookies: 0, aead_algorithms: ["AEAD_AES_SIV_CMAC_256", "UNKNOWN", "AEAD_AES_128_GCM_SIV"]}}
  end
end
